/*
 * Copyright 2005-2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.springframework.ldap.support;

import org.junit.Test;
import org.springframework.ldap.BadLdapGrammarException;

import static org.junit.Assert.assertEquals;

/**
 * Unit test for the LdapEncode class.
 * 
 * @author Adam Skogman
 */
public class LdapEncoderTest  {

    @Test
    public void testFilterEncode() {
        String correct = "\\2aa\\2ab\\28c\\29d\\2a\\5c";
        assertEquals(correct, LdapEncoder.filterEncode("*a*b(c)d*\\"));

    }

    @Test
    public void testNameEncode() {

        String res = LdapEncoder.nameEncode("# foo ,+\"\\<>; ");

        assertEquals("\\# foo \\,\\+\\\"\\\\\\<\\>\\;\\ ", res);
    }

    @Test
    public void testNameDecode() {

        String res = LdapEncoder
                .nameDecode("\\# foo \\,\\+\\\"\\\\\\<\\>\\;\\ ");

        assertEquals("# foo ,+\"\\<>; ", res);
    }

    @Test(expected = BadLdapGrammarException.class)
    public void testNameDecode_slashlast() {
        LdapEncoder.nameDecode("\\");
    }

}
